//=============================================================================
//  MuseScore
//
//  Test plugin
//
//  Copyright (C)2011 by Gideon Raveh
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License version 2.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//=============================================================================

var mscorePlugin = 
{
	majorVersion: 1,
    minorVersion: 1,
    menu: 'Plugins.Expand Chords',
    init: init,
    run:  run
};

var	chords = new Array(); 
var	notes = new Array(); 
var	steps = new Array(9, 11, 0, 2, 4, 5, 7);

mscorePlugin;

function init()
{
}

function run()
{
	readTable();
	readXML();
	readScore();
}

/*============================================  Read Table ========================================*/
function readTable()
{
	var chordFile = new QFile("styles/chords.xml");
    chordFile.open(QIODevice.ReadOnly);
	chordStream = new QTextStream(chordFile);
	while (!chordStream.atEnd())
	{
		line = chordStream.readLine();
 		if (line.match('chord id'))
			break;
	}	
	
	while (!chordStream.atEnd()) 
	{
		readEntry();
		line = chordStream.readLine();
	}
}

function readEntry()
{
	var chord = new Object();
	
	var p1 = line.indexOf('id="') + 4;
	var p2 = line.indexOf('"', p1);
	var id = line.substr(p1, p2 - p1);
	chord.id = id;	
	
	while (!chordStream.atEnd()) 
	{
		line = chordStream.readLine();
 		if (line.match('name')) 
			chord.name = decode('name');
 		else if (line.match('voicing')) 
			chord.voicing = decode('voicing');
 		else if (line.match('/chord')) 
			break;
	}
	chords[id] = chord;
}

/*============================================  Read XML file  ========================================*/
function readXML()
{
    var tempPath = QDir.tempPath(); 
	var xmlPath = tempPath + "/score.xml";	
	curScore.save(xmlPath,"xml");
	 
	var xmlFile = new QFile(xmlPath);
    xmlFile.open(QIODevice.ReadOnly);
	xmlStream = new QTextStream(xmlFile);
	while (!xmlStream.atEnd()) 
	{
		line = xmlStream.readLine();
 		if (line.match('divisions')) 
			divisions = decode('divisions');
 		if (line.match('harmony') || line.match('note')) 
			readNext();
	}
}

function readNext()
{
	var note = new Object;
	if (line.match('harmony')) 
	{
		note.harmony = readHarmony();
		line = xmlStream.readLine();
	}
	if (line.match('note'))
	{	
		readNote(note);
		notes.push(note);
	}
}

function readHarmony()
{
	var harmony = new Object; 	
	while (!xmlStream.atEnd()) 
	{
		line = xmlStream.readLine();
 		if (line.match('root-step')) 
			harmony.root = decode('root-step');
 		else if (line.match('kind')) 
		{
			var p1 = line.indexOf('text="') + 6;
			var p2 = line.indexOf('"', p1);
			harmony.text = line.substr(p1, p2 - p1);
			harmony.kind = decode('kind');
		}
 		else if (line.match('/harmony')) 
			break;
	}
	return harmony;
}

function readNote(note)
{
	while (!xmlStream.atEnd()) 
	{
		line = xmlStream.readLine();
 		if (line.match('rest')) 
			note.isRest = 'true';
 		else if (line.match('step')) 
			note.step = decode('step');
 		else if (line.match('alter')) 
			note.alter = new Number(decode('alter'));
 		else if (line.match('octave')) 
			note.octave = new Number(decode('octave'));
 		else if (line.match('duration')) 
			note.duration = decode('duration');
		else if (line.match('/note')) 
			return;
	}
}

function decode(txt)
{
	var p1 = line.indexOf('>') + 1;
	var p2 = line.indexOf('<', p1);
	return line.substr(p1, p2 - p1);
}

/*============================================  Read Score  ========================================*/
function readScore()
{
	var trebleCursor = new Cursor(curScore);
	trebleCursor.staff = 1;
	trebleCursor.rewind();
	
	var bassCursor = new Cursor(curScore);
	bassCursor.staff = 2;
	bassCursor.rewind();

    for (i = 0; i < notes.length; i++)
    {
        var xmlNote = notes[i];
        var tickLen = xmlNote.duration * 480 / divisions;
        if(xmlNote.isRest)
		{
			var rest = new Rest(curScore);
            rest.tickLen = tickLen;
            print("Rest "+rest.tickLen);
		}
        else
		{
			var chord = new Chord(curScore);
			var note = new Note(curScore);
            note.pitch = getPitch(xmlNote);
            chord.addNote(note);
            chord.tickLen = tickLen;
            print(xmlNote.step+" "+note.pitch+" "+chord.tickLen);
		}
    }
}
    
function getPitch(xmlNote)
{
    var i = xmlNote.step.charCodeAt(0) - 65;
    var pitch = steps[i];
    if(xmlNote.alter)
        pitch += xmlNote.alter;
    var octave = (xmlNote.octave + 1) * 12;
    pitch += octave;
    return pitch;
}

